
/*
   This program will make an alphabetical list of words that are
   found in user input.  A word here means a sequence of letters.
   A word can also include an apostrophe if it is between two letters,
   as in the word "can't".  The program will read input until it
   encounters a period (".").  Other non-letters in the input will
   be ignored.  (The program will crash if there are more than
   1000 different words in the input.)
*/


public class WordList {

   
   public static void main(String[] args) {

      String[] wordList;  // An array to contain the words.
                          //    Words are converted to lowercase before
                          //    they are put in the array.  Each word is
                          //    only stored once in the array.
      int wordCt;         // The number of words that are currently
                          //    stored in the array.
                          
      wordList = new String[1000];  // Allow for up to 1000 words.
      wordCt = 0;                   // Start with 0 words.

      System.out.println("\nType as many words as you like, on as many lines as");
      System.out.println("you like, ending with a period.  You can include");
      System.out.println("other punctuation, but it will be ignored.  Anything");
      System.out.println("after the '.' will be ignored.  A list of the words");
      System.out.println("that you entered will be printed in alphabetical");
      System.out.println("order.  The words will be changed to all lower case");
      System.out.println("and any duplicates will be removed.");
      System.out.println("\nStart typing!\n");

      while (true) {  
            // Read and process one word.
         skipJunk();
         if (TextIO.peek() == '.')
            break; // End when a period is found.
         String w = readWord().toLowerCase();
         if ( ! find(w, wordList, wordCt) ) {
              // If the word is not already in the list, add it.
            wordList[wordCt++] = w;
         }
      }

      sort(wordList, wordCt);
      System.out.println("\n\nThe words you entered were:\n");
      print(wordList, wordCt);
      System.out.println();

   } // end main()
   

   static void skipJunk() {
        // Skip past "junk" characters in the input.  Everything
        // except a letter or a period is considered.  When this
        // routine returns, the next character of input is either
        // a letter or a period.
      while (true) {
         char ch = TextIO.peek();  // Look ahead to next char in input.
         if (ch == '.')
            return;
         if (Character.isLetter(TextIO.peek()))
            return;
         TextIO.getAnyChar();  // Read the char.
      }
   }
      

   static String readWord() {
        // This subroutine assumes that the next character in the
        // input is a letter.  It will read and return one word.
        // A word is a series of letters and can also include an
        // apostrophe if it is between two letters.
      String word = "";  // The word.
      while (true) {
         char ch = TextIO.peek(); // Look ahead to next char in input.
         if (Character.isLetter(ch)) {
            word += TextIO.getAnyChar();  // Add a letter to the word.
         }
         else if (ch == '\'') {
              // To process an apostrophe, look at the character that
              // follows it.  If it is a letter, add both the apostrophe
              // and the letter that follows it to the word.
            TextIO.getAnyChar();  // Read the apostrophe.
            ch = TextIO.peek();   // The char that follows the apostrophe.
            if (Character.isLetter(ch))
               word += ("\'" + TextIO.getAnyChar());
            else
               break; // The apostrophe is followed by a non-letter.
         }
         else {
            break; // The next char is not a letter or an apostrophe
         }
      }
      return word;
   }
   

   static boolean find(String word, String[] list, int size) {
         // Search for the word in the array.  The size parameter
         // gives the number of positions in the array that
         // are in use.  This routine looks for the word in
         // list[0], list[1], list[2], ..., list[size-1].  The return
         // value is true if the word is found, false if not.
      for (int i = 0; i < size; i++) {
         if (word.equals(list[i]))
            return true;
      }
      return false;
   }
   

   static void sort(String[] A, int count) {
         // Sort the words in A, in positions 0, 1, ..., count-1, into
         // alphabetical order.  This uses a standard insertion sort.
      for (int top = 1; top < count; top++) {
         String w = A[top];
         int loc = top - 1;
         while (loc >= 0 && A[loc].compareTo(w) > 0) {
            A[loc+1] = A[loc];
            loc--;
         }
         A[loc+1] = w;
      }
   }
   

   static void print(String[] words, int wordsInList) {
        // Print the strings in the array in positions 0, 1, ...,
        // wordsInList.  Strings are printed one to a line.
      for (int index = 0; index < wordsInList; index++)
         System.out.println("   " + words[index]);
   }


} // end class WordList
